/*
 * Bg.java
 *
 * Last modified: <2008/08/29 15:57:20 +0900> By SUMIDA
 */
package game;

import java.awt.Graphics;

public class Bg {

    /* BG数 */
    public static byte BG_MAX = 4;

    /* マップサイズ */
    public static byte BG_MAP_SIZE    = 8;
    public static byte BG_MAP_WIDTH   = (byte)(SystemInfo.WIDTH / BG_MAP_SIZE);
    public static byte BG_MAP_HEIGHT  = (byte)(SystemInfo.HEIGHT / BG_MAP_SIZE);
    public static byte BG_MAP_RESERVE = 2;

    public static class MapAttr {
        public int surface_id;
        public int map_id;
    }

    public static class Info {

        //public int priority;
        public int ofs_x;
        public int ofs_y;
        public int font_x;
        public int font_y;
        public int frame_x;
        public int frame_y;
        public int frame_w;
        public int frame_h;

        MapAttr[][] map;

        public Info() {
            map = new MapAttr[BG_MAP_WIDTH + BG_MAP_RESERVE][BG_MAP_HEIGHT + BG_MAP_RESERVE];
            for (int i = 0; i < BG_MAP_WIDTH + BG_MAP_RESERVE; i++) {
                for (int j = 0; j < BG_MAP_HEIGHT + BG_MAP_RESERVE; j++) {
                    map[i][j] = new MapAttr();
                }
            }
        }
    }

    public static Info[] info = new Info[BG_MAX];

    public static void init() {

        System.out.print("bg_init ... ");

        for (int i = 0; i < info.length; i++) {

            Info it = new Info();
            info[i] = it;

            //it.priority = i * 256 / BG_MAX;
            it.ofs_x   = 8;
            it.ofs_y   = 8;
            it.font_x  = 0;
            it.font_y  = 0;
            it.frame_x = 1;
            it.frame_y = 1;
            it.frame_w = BG_MAP_WIDTH;
            it.frame_h = BG_MAP_HEIGHT;

            /*
            for (MapAttr attr: it.map) {
                attr.surface_id = 0;
                attr.map_id     = 0;
            }
            */
        }

        System.out.print("done.\n");
    }

    public static void rendaring(Graphics screen, int bg_no) {

        Info it = info[bg_no];
        int col, row;
        int ofs_x, ofs_y;
        int cmax, rmax;
        int co, ro;

        it.ofs_x %= (BG_MAP_WIDTH  + BG_MAP_RESERVE) * BG_MAP_SIZE;
        it.ofs_y %= (BG_MAP_HEIGHT + BG_MAP_RESERVE) * BG_MAP_SIZE;
        col = it.ofs_x >> 3;
        row = it.ofs_y >> 3;
        ofs_x = (col << 3) - it.ofs_x;
        ofs_y = (row << 3) - it.ofs_y;
        cmax = (ofs_x == 0)? BG_MAP_WIDTH : BG_MAP_WIDTH  + 1;
        rmax = (ofs_y == 0)? BG_MAP_HEIGHT: BG_MAP_HEIGHT + 1;

        for (ro = 0; ro < rmax; ro++) {
            int r = (BG_MAP_HEIGHT + BG_MAP_RESERVE + row + ro) % (BG_MAP_HEIGHT + BG_MAP_RESERVE);
            int y = ofs_y + (ro << 3);

            for (co = 0; co < cmax; co++) {
                int c = (BG_MAP_WIDTH + BG_MAP_RESERVE + col + co) % (BG_MAP_WIDTH + BG_MAP_RESERVE);
                MapAttr attr = it.map[c][r];
                if (attr.map_id > 0) {
                    int x = ofs_x + (co << 3);
                    Surface.drawFrame8(screen, attr.surface_id, attr.map_id, x, y);
                }
            }
        }
    }

    public static void fontFrame(int bg_no, int x, int y, int w, int h) {

        Info it = info[bg_no];

        it.frame_x = x;
        it.frame_y = y;
        it.frame_w = w;
        it.frame_h = h;
    }

    public static void fontLocate(int bg_no, int x, int y) {

        Info it = info[bg_no];

        it.font_x = x;
        it.font_y = y;
    }

    public static void fontPrint(int bg_no,  String msg) {

        Info it = info[bg_no];

        for (int i = 0; it.font_y < it.frame_h && i < msg.length(); i++) {

            char ch = msg.charAt(i);

            switch (ch) {
            case ' ':
                it.font_x++;
                break;
            case '\t':
                it.font_x += 4;
                break;
            case '\n':
                it.font_x = 0;
                it.font_y++;
                break;
            default:
                it.map[it.font_x + it.frame_x][it.font_y + it.frame_y].map_id = ch;
                it.font_x++;
                break;
            }

            if (it.font_x >= it.frame_w) {
                it.font_x = 0;
                it.font_y++;
            }
        }
    }
}

