/*
 * Sample2.java
 *
 * Last modified: <2008/08/29 16:51:49 +0900> By SUMIDA
 */
package game;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.DisplayMode;
import java.awt.Graphics;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.image.BufferedImage;
import java.text.DecimalFormat;
import javax.swing.*;

/**
 * シューティング・ゲーム　サンプル
 */
public class Sample2 extends JPanel implements Runnable {

    /** serialVersionUID */
    private static final long serialVersionUID = 1L;

    private static JFrame frame;
    private static GraphicsDevice device;

    // ダブルバッファリング（db）用
    private Image dbImage;

    // スコア表示フォーマット
    DecimalFormat scf = new DecimalFormat("00000000");

    // レベル表示フォーマット
    DecimalFormat lvf = new DecimalFormat("00");


    public Sample2() {

        // パネルサイズを設定
        setPreferredSize(new Dimension(SystemInfo.WIDTH, SystemInfo.HEIGHT));

        // パネルがキーボードを受け付けるようフォーカスを受け取る
        setFocusable(true);

        // キーリスナを登録
        addKeyListener(Input.instance);
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
    }

    /**
     * 初期化処理。
     */
    private void start() {

        dbImage = createImage(SystemInfo.WIDTH, SystemInfo.HEIGHT);

        Surface.init(this);
        Sprite.init();
        Bg.init();
        Sound.init();

        /* テクスチャ関連ファイルの設定 */
        Surface.load("image/font8x8.png", Texture.TEX_FONT);
        Surface.load("image/bg.png", Texture.TEX_BG);

        Surface.load("image/title.png", Texture.TEX_TITLE);
        Surface.load("image/gameover.png", Texture.TEX_GAMEOVER);
        Surface.load("image/pause.png", Texture.TEX_PAUSE);

        Surface.load("image/explosion.png", Texture.TEX_EXPLOSION);
        Surface.setFrame(Texture.TEX_EXPLOSION, 64, 64);
        Surface.load("image/bomb.png", Texture.TEX_BOMB);
        Surface.setFrame(Texture.TEX_BOMB, 240, 240);
        Surface.load("image/fire.png", Texture.TEX_FIRE);
        Surface.setFrame(Texture.TEX_FIRE, 56, 56);

        Surface.load("image/ship.png", Texture.TEX_SHIP);
        Surface.setFrame(Texture.TEX_SHIP, 60, 60);
        Surface.load("image/hit.png", Texture.TEX_HIT);
        Surface.setFrame(Texture.TEX_HIT, 32, 32);

        Surface.load("image/enemy1.png", Texture.TEX_ENEMY1);
        Surface.setFrame(Texture.TEX_ENEMY1, 60, 60);
        Surface.load("image/enemy2.png", Texture.TEX_ENEMY2);
        Surface.setFrame(Texture.TEX_ENEMY2, 60, 60);
        Surface.load("image/enemy3.png", Texture.TEX_ENEMY3);
        Surface.setFrame(Texture.TEX_ENEMY3, 60, 60);
        Surface.load("image/enemy4.png", Texture.TEX_ENEMY4);
        Surface.setFrame(Texture.TEX_ENEMY4, 60, 60);
        Surface.load("image/enemy5.png", Texture.TEX_ENEMY5);
        Surface.setFrame(Texture.TEX_ENEMY5, 60, 60);
        Surface.load("image/missile.png", Texture.TEX_MISSILE);
        Surface.setFrame(Texture.TEX_MISSILE, 16, 45);

        Surface.load("image/boss1.png", Texture.TEX_BOSS1);
        Surface.setFrame(Texture.TEX_BOSS1, 100, 181);
        Surface.load("image/boss2.png", Texture.TEX_BOSS2);
        Surface.setFrame(Texture.TEX_BOSS2, 100, 150);

        Surface.load("image/shot.png", Texture.TEX_SHOT);
        Surface.setFrame(Texture.TEX_SHOT, 28, 16);
        Surface.load("image/ballet.png", Texture.TEX_BALLET);
        Surface.setFrame(Texture.TEX_BALLET, 8, 8);
        Surface.load("image/ray1.png", Texture.TEX_RAY1);
        Surface.load("image/ray2.png", Texture.TEX_RAY2);
        Surface.load("image/photon.png", Texture.TEX_PHOTON);
        Surface.setFrame(Texture.TEX_PHOTON, 24, 24);

        Surface.load("image/moon.png", Texture.TEX_MOON);
        Surface.load("image/planet.png", Texture.TEX_PLANET);

        /* 音楽関連ファイルの設定 */
        Sound.loadMusic("sound/50la-campanella.mid", Audio.BGM_TITLE);
        Sound.loadMusic("sound/12amaironokaminootome.mid", Audio.BGM_01);
        Sound.loadMusic("sound/8maou.mid", Audio.BGM_02);
        Sound.loadMusic("sound/272morobitokozorite_b.mid", Audio.BGM_OVER);
        Sound.loadEffect("sound/tm2_shoot000.wav", Audio.SE_SHOT0);
        Sound.loadEffect("sound/tm2_shoot001.wav", Audio.SE_SHOT1);
        Sound.loadEffect("sound/tm2_sonic000.wav", Audio.SE_SHOT2);
        Sound.loadEffect("sound/tm2_bom001.wav"  , Audio.SE_BOMB0);
        Sound.loadEffect("sound/tm2r_bom26_c.wav", Audio.SE_BOMB1);
        Sound.loadEffect("sound/tm2r_bom31_b.wav", Audio.SE_BOMB2);

        /* タイトル表示設定 */
        SystemInfo.status |= SystemInfo.SYSTEM_MODE_READY;

        /* 効果マネージャの初期化 */
        Effect.init();

        /* 自機の設定 */
        MyShip.init();
        MyShip.ship.stat = Ship.SHIP_STAT_LOST;

        /* 敵機管理の設定 */
        Enemy.init();

        /* マップの設定 */
        Map.init();

        /* 背景の設定 */
        background_init();

        /* BGMの再生 */
        Sound.playMusic(Audio.BGM_TITLE, -1);
    }

    /**
     * 終了処理。
     */
    private void finish() {
        Input.free();
        Surface.free();
        Sound.free();
    }

    /**
     * 更新処理。
     */
    private void update() {
        /* 入力装置からの読み込み */
        Input.read();

        if ((Input.input[Input.P1] & Input.INPUT_KEYDOWN_START) != 0) {
            int mode = SystemInfo.status & SystemInfo.SYSTEM_MODE_MASK;

            switch (mode) {
            case SystemInfo.SYSTEM_MODE_READY:
            case SystemInfo.SYSTEM_MODE_OVER:
                /* ゲーム開始、再開 */
                restart();
                break;

            case SystemInfo.SYSTEM_MODE_PAUSE:
                /* ポーズ機能の解除 */
                SystemInfo.status &= ~SystemInfo.SYSTEM_MODE_PAUSE;
                break;

            default:
                /* ポーズ機能の設定 */
                SystemInfo.status |= SystemInfo.SYSTEM_MODE_PAUSE;
                break;
            }
        }

        if ((SystemInfo.status & SystemInfo.SYSTEM_MODE_MASK) != SystemInfo.SYSTEM_MODE_PAUSE) {

            /* 表示スプライトを破棄 */
            Sprite.clean();

            /* 背景のスクロール */
            Bg.info[1].ofs_y -= 2;
            Bg.info[2].ofs_y -= 1;

            /* マップの制御 */
            if ((SystemInfo.status & SystemInfo.SYSTEM_MODE_MASK) == 0) {
                Map.action();
            }

            /* 自機ショットの制御 */
            SystemInfo.score += Shot.action(MyShip.ship_shot_mgr, Enemy.enemy_mgr);

            /* 敵機ショットの制御 */
            SystemInfo.score += Shot.action(Enemy.enemy_shot_mgr, MyShip.ship_mgr);

            /* 自機の制御 */
            MyShip.action();

            /* 敵機の制御 */
            Enemy.action();

            /* 画面効果の制御 */
            Effect.action();

            /* スコア表示 */
            Bg.fontLocate(0, 1, 1);
            Bg.fontPrint(0, "SCORE:" + scf.format(SystemInfo.score) + "  LEVEL:" + lvf.format(SystemInfo.level));
        }
    }

    /**
     * バッファにレンダリング（ダブルバッファリング）
     */
    private void render() {
        // バッファイメージの描画オブジェクト
        if (dbImage != null) {
            Graphics screen = dbImage.getGraphics();

            // バッファをクリアする
            screen.setColor(Color.WHITE);
            screen.fillRect(0, 0, SystemInfo.WIDTH, SystemInfo.HEIGHT);

            // バッファへ描画する
            Bg.rendaring(screen, 3);
            Bg.rendaring(screen, 2);
            Bg.rendaring(screen, 1);
            Sprite.rendering(screen);
            Bg.rendaring(screen, 0);
            systemInfoDraw(screen);

            screen.dispose();
        }
    }

    /**
     * バッファを画面に描画。
     */
    private void draw() {

        // グラフィックオブジェクトを取得
        Graphics g = getGraphics();

        // バッファイメージを画面に描画
        g.drawImage(dbImage, 0, 0, null);

        Toolkit.getDefaultToolkit().sync();

        // グラフィックオブジェクトを破棄
        g.dispose();
    }

    /**
     * 再スタート処理。
     */
    private void restart() {
        SystemInfo.status = 0;
        SystemInfo.score  = 0;
        SystemInfo.frame  = 0;

        MyShip.init();
        Enemy.init();
        Map.init();
        Effect.init();

        Sound.playMusic(Audio.BGM_01, -1);
    }

    /**
     * 背景の初期化。
     */
    private void background_init() {

        for (int y = 0; y < Bg.BG_MAP_HEIGHT + Bg.BG_MAP_RESERVE; y++) {
            for (int x = 0; x < Bg.BG_MAP_WIDTH + Bg.BG_MAP_RESERVE; x++) {
                Bg.info[1].map[x][y].surface_id = 1;
                Bg.info[1].map[x][y].map_id = (int)SystemInfo.Random(3, 9);
                Bg.info[2].map[x][y].surface_id = 1;
                Bg.info[2].map[x][y].map_id = (int)SystemInfo.Random(2, 6);
                Bg.info[3].map[x][y].surface_id = 1;
                Bg.info[3].map[x][y].map_id = 1;
            }
        }
    }


    /** 看板ID */
    private static final int[] info_id = {-1, Texture.TEX_TITLE, Texture.TEX_GAMEOVER, Texture.TEX_PAUSE};

    /**
     * システム情報の描画
     */
    private void systemInfoDraw(Graphics screen) {

        int id = info_id[SystemInfo.status & SystemInfo.SYSTEM_MODE_MASK];

        if (id >= 0) {
            Surface.Info surface = Surface.getInfo(id);
            Surface.draw(screen, id,
                         (SystemInfo.WIDTH - surface.frame_width) / 2,
                         (SystemInfo.HEIGHT - surface.frame_height) / 2);
        }
    }

    /**
     * メイン処理。
     */
    public void run() {
        long beforeTime, afterTime, timeDiff, sleepTime;
        long overSleepTime = 0L;
        int noDelays = 0;

        try {
            // 初期化
            start();

            beforeTime = System.nanoTime();
            Fps.start(beforeTime);

            while (isRunning()) {

                // 更新処理
                update();

                // レンダリング
                render();

                // 画面更新
                draw();

                afterTime = System.nanoTime();
                timeDiff = afterTime - beforeTime;

                // 前回のフレームの休止時間誤差も引いておく
                sleepTime = (SystemInfo.DEF_FPS_PERIOD - timeDiff) - overSleepTime;

                if (sleepTime > 0) {
                    // 休止時間がとれる場合
                    try {
                        // nano->ms
                        Thread.sleep(sleepTime / 1000000L);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    // sleep()の誤差
                    overSleepTime = (System.nanoTime() - afterTime) - sleepTime;

                } else {
                    // 状態更新・レンダリングで時間を使い切ってしまい
                    // 休止時間がとれない場合
                    overSleepTime = 0L;
                    // 休止なしが16回以上続いたら
                    if (++noDelays >= 16) {
                        Thread.yield(); // 他のスレッドを強制実行
                        noDelays = 0;
                    }
                }

                beforeTime = System.nanoTime();

                // FPSを計算
                Fps.calc();
            }

            // 終了処理
            finish();

        } catch (Exception e) {
            e.printStackTrace();

        } finally {
            restoreScreen();
            System.exit(0);
        }
    }

    /**
     * フルスクリーンモードの設定処理。
     */
    private static void initFullScreen() {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        device = ge.getDefaultScreenDevice();

        if (!device.isFullScreenSupported()) {
            System.out.println("フルスクリーンモードはサポートされていません。");
            System.exit(0);
        }

        // タイトルバー・ボーダー非表示
        frame.setUndecorated(true);

        // フルスクリーン化
        device.setFullScreenWindow(frame);

        // ディスプレイモードの変更
        DisplayMode mode = new DisplayMode(SystemInfo.WIDTH,
                SystemInfo.HEIGHT,
                SystemInfo.DEPTH,
                DisplayMode.REFRESH_RATE_UNKNOWN);
        device.setDisplayMode(mode);

        // マウスカーソルを消す
        Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(
                new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB),
                new Point(), "");
        frame.setCursor(cursor);

        // paintイベントを無効化
        frame.setIgnoreRepaint(true);
    }

    /**
     * フルスクリーンモードの後処理。
     */
    private static void restoreScreen() {
        if (device != null) {
            Window w = device.getFullScreenWindow();
            if (w != null) {
                w.dispose();
            }
            device.setFullScreenWindow(null);
        }
    }

    /**
     * ウィンドウ表示設定処理。
     */
    private static void initWindow() {
        frame.pack();
        frame.setResizable(false);
    }

    /**
     * 起動中判定処理。
     * @return 判定結果 ウィンドウが開いている、終了状態でない場合にtrue
     */
    private static synchronized boolean isRunning() {
        //System.System.out.println("isRunning=" + frame.isVisible());
        return (frame.isVisible() && Input.quit == false);
    }

    /**
     * プログラムのエントリポイント
     * @param  argvs  プログラムの引数
     */
    public static void main(String[] args) {

        // システム情報を初期化
        SystemInfo.init();

        // オプション情報を読み込む
        SystemInfo.readArgs(args);

        // デバッグ設定
        //SystemInfo.options |= SystemInfo.SYSTEM_OPT_VERBOSE;

        // 描画エリアのコンテナとなるフレームを生成
        frame = new JFrame();

        // 実際にリアルタイムに動作するオブジェクトを生成
        Sample2 game = new Sample2();

        // 画面表示用のフレームを準備
        //frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.getContentPane().add(game);

        if ((SystemInfo.options & SystemInfo.SYSTEM_OPT_FULLSCREEN) != 0) {
            // フルスクリーン設定
            initFullScreen();
        } else {
            // ウィンドウ設定
            initWindow();
        }

        // 表示
        frame.setTitle(SystemInfo.TITLE);
        frame.setVisible(true);

        // スレッドの動作開始
        (new Thread(game)).start();
    }
}
